# r e g i o n   P D F s h a r p   -   A   . N E T   l i b r a r y   f o r   p r o c e s s i n g   P D F 
 
 / / 
 
 / /   A u t h o r s : 
 
 / /       S t e f a n   L a n g e   ( m a i l t o : S t e f a n . L a n g e @ p d f s h a r p . c o m ) 
 
 / / 
 
 / /   C o p y r i g h t   ( c )   2 0 0 5 - 2 0 0 9   e m p i r a   S o f t w a r e   G m b H ,   C o l o g n e   ( G e r m a n y ) 
 
 / / 
 
 / /   h t t p : / / w w w . p d f s h a r p . c o m 
 
 / /   h t t p : / / s o u r c e f o r g e . n e t / p r o j e c t s / p d f s h a r p 
 
 / / 
 
 / /   P e r m i s s i o n   i s   h e r e b y   g r a n t e d ,   f r e e   o f   c h a r g e ,   t o   a n y   p e r s o n   o b t a i n i n g   a 
 
 / /   c o p y   o f   t h i s   s o f t w a r e   a n d   a s s o c i a t e d   d o c u m e n t a t i o n   f i l e s   ( t h e   " S o f t w a r e " ) , 
 
 / /   t o   d e a l   i n   t h e   S o f t w a r e   w i t h o u t   r e s t r i c t i o n ,   i n c l u d i n g   w i t h o u t   l i m i t a t i o n 
 
 / /   t h e   r i g h t s   t o   u s e ,   c o p y ,   m o d i f y ,   m e r g e ,   p u b l i s h ,   d i s t r i b u t e ,   s u b l i c e n s e , 
 
 / /   a n d / o r   s e l l   c o p i e s   o f   t h e   S o f t w a r e ,   a n d   t o   p e r m i t   p e r s o n s   t o   w h o m   t h e 
 
 / /   S o f t w a r e   i s   f u r n i s h e d   t o   d o   s o ,   s u b j e c t   t o   t h e   f o l l o w i n g   c o n d i t i o n s : 
 
 / / 
 
 / /   T h e   a b o v e   c o p y r i g h t   n o t i c e   a n d   t h i s   p e r m i s s i o n   n o t i c e   s h a l l   b e   i n c l u d e d 
 
 / /   i n   a l l   c o p i e s   o r   s u b s t a n t i a l   p o r t i o n s   o f   t h e   S o f t w a r e . 
 
 / / 
 
 / /   T H E   S O F T W A R E   I S   P R O V I D E D   " A S   I S " ,   W I T H O U T   W A R R A N T Y   O F   A N Y   K I N D ,   E X P R E S S   O R 
 
 / /   I M P L I E D ,   I N C L U D I N G   B U T   N O T   L I M I T E D   T O   T H E   W A R R A N T I E S   O F   M E R C H A N T A B I L I T Y , 
 
 / /   F I T N E S S   F O R   A   P A R T I C U L A R   P U R P O S E   A N D   N O N I N F R I N G E M E N T .   I N   N O   E V E N T   S H A L L 
 
 / /   T H E   A U T H O R S   O R   C O P Y R I G H T   H O L D E R S   B E   L I A B L E   F O R   A N Y   C L A I M ,   D A M A G E S   O R   O T H E R 
 
 / /   L I A B I L I T Y ,   W H E T H E R   I N   A N   A C T I O N   O F   C O N T R A C T ,   T O R T   O R   O T H E R W I S E ,   A R I S I N G 
 
 / /   F R O M ,   O U T   O F   O R   I N   C O N N E C T I O N   W I T H   T H E   S O F T W A R E   O R   T H E   U S E   O R   O T H E R   
 
 / /   D E A L I N G S   I N   T H E   S O F T W A R E . 
 
 # e n d r e g i o n 
 
 
 
 u s i n g   S y s t e m ; 
 
 u s i n g   S y s t e m . D i a g n o s t i c s ; 
 
 u s i n g   S y s t e m . C o l l e c t i o n s ; 
 
 u s i n g   S y s t e m . T e x t ; 
 
 u s i n g   S y s t e m . I O ; 
 
 u s i n g   P d f S h a r p . P d f ; 
 
 u s i n g   P d f S h a r p . P d f . A d v a n c e d ; 
 
 u s i n g   P d f S h a r p . D r a w i n g ; 
 
 u s i n g   P d f S h a r p . F o n t s . O p e n T y p e ; 
 
 u s i n g   P d f S h a r p . P d f . F i l t e r s ; 
 
 u s i n g   P d f S h a r p . I n t e r n a l ; 
 
 
 
 n a m e s p a c e   P d f S h a r p . P d f . A d v a n c e d 
 
 { 
 
     / / /   < s u m m a r y > 
 
     / / /   R e p r e s e n t s   a   C I D F o n t   d i c t i o n a r y . 
 
     / / /   < / s u m m a r y > 
 
     i n t e r n a l   c l a s s   P d f C I D F o n t   :   P d f F o n t 
 
     { 
 
         p u b l i c   P d f C I D F o n t ( P d f D o c u m e n t   d o c u m e n t ) 
 
             :   b a s e ( d o c u m e n t ) 
 
         {   } 
 
 
 
         p u b l i c   P d f C I D F o n t ( P d f D o c u m e n t   d o c u m e n t ,   P d f F o n t D e s c r i p t o r   f o n t D e s c r i p t o r ,   X F o n t   f o n t ) 
 
             :   b a s e ( d o c u m e n t ) 
 
         { 
 
             E l e m e n t s . S e t N a m e ( K e y s . T y p e ,   " / F o n t " ) ; 
 
             E l e m e n t s . S e t N a m e ( K e y s . S u b t y p e ,   " / C I D F o n t T y p e 2 " ) ; 
 
             / /   T h i s   c o d e   i s   f o o b a r .   A   P D F   l i t e r a l   c o n t a i n i n g   s t r i n g s   i s   n o t   e n c r y p t e d ,   b u t   d e c r y p e d   w h e n   o p e n i n g   t h e   P D F   f i l e .   
 
             / / E l e m e n t s . S e t V a l u e ( K e y s . C I D S y s t e m I n f o , 
 
             / /     n e w   P d f L i t e r a l ( " < <   / O r d e r i n g   ( I d e n t i t y )   / R e g i s t r y   ( A d o b e )   / S u p p l e m e n t   0 > > " ) ) ; 
 
             / /   M u s t   u s e   r e a l   o b j e c t s   b e c a u s e   P D F   l i t e r a l s   c o n t a i n i n g   s t r i n g s   i s   n o t   e n c r y p t e d .   T h a t   c a u s e s   a n   e r r o r   
 
             / /   w h e n   t h e   f i l e   i s   d e c r y p t e d . 
 
             P d f D i c t i o n a r y   c i d   =   n e w   P d f D i c t i o n a r y ( ) ; 
 
             c i d . E l e m e n t s . S e t S t r i n g ( " / O r d e r i n g " ,   " I d e n t i t y " ) ; 
 
             c i d . E l e m e n t s . S e t S t r i n g ( " / R e g i s t r y " ,   " A d o b e " ) ; 
 
             c i d . E l e m e n t s . S e t I n t e g e r ( " / S u p p l e m e n t " ,   0 ) ; 
 
             E l e m e n t s . S e t V a l u e ( K e y s . C I D S y s t e m I n f o ,   c i d ) ; 
 
 
 
             t h i s . f o n t D e s c r i p t o r   =   f o n t D e s c r i p t o r ; 
 
             O w n e r . i r e f T a b l e . A d d ( f o n t D e s c r i p t o r ) ; 
 
             E l e m e n t s [ K e y s . F o n t D e s c r i p t o r ]   =   f o n t D e s c r i p t o r . R e f e r e n c e ; 
 
 
 
             F o n t E n c o d i n g   =   f o n t . P d f O p t i o n s . F o n t E n c o d i n g ; 
 
             F o n t E m b e d d i n g   =   f o n t . P d f O p t i o n s . F o n t E m b e d d i n g ; 
 
         } 
 
 
 
         p u b l i c   P d f C I D F o n t ( P d f D o c u m e n t   d o c u m e n t ,   P d f F o n t D e s c r i p t o r   f o n t D e s c r i p t o r ,   b y t e [ ]   f o n t D a t a ) 
 
             :   b a s e ( d o c u m e n t ) 
 
         { 
 
             E l e m e n t s . S e t N a m e ( K e y s . T y p e ,   " / F o n t " ) ; 
 
             E l e m e n t s . S e t N a m e ( K e y s . S u b t y p e ,   " / C I D F o n t T y p e 2 " ) ; 
 
             P d f D i c t i o n a r y   c i d   =   n e w   P d f D i c t i o n a r y ( ) ; 
 
             c i d . E l e m e n t s . S e t S t r i n g ( " / O r d e r i n g " ,   " I d e n t i t y " ) ; 
 
             c i d . E l e m e n t s . S e t S t r i n g ( " / R e g i s t r y " ,   " A d o b e " ) ; 
 
             c i d . E l e m e n t s . S e t I n t e g e r ( " / S u p p l e m e n t " ,   0 ) ; 
 
             E l e m e n t s . S e t V a l u e ( K e y s . C I D S y s t e m I n f o ,   c i d ) ; 
 
 
 
             t h i s . f o n t D e s c r i p t o r   =   f o n t D e s c r i p t o r ; 
 
             O w n e r . i r e f T a b l e . A d d ( f o n t D e s c r i p t o r ) ; 
 
             E l e m e n t s [ K e y s . F o n t D e s c r i p t o r ]   =   f o n t D e s c r i p t o r . R e f e r e n c e ; 
 
 
 
             F o n t E n c o d i n g   =   P d f F o n t E n c o d i n g . U n i c o d e ; 
 
             F o n t E m b e d d i n g   =   P d f F o n t E m b e d d i n g . A l w a y s ; 
 
         } 
 
 
 
         p u b l i c   s t r i n g   B a s e F o n t 
 
         { 
 
             g e t   {   r e t u r n   E l e m e n t s . G e t N a m e ( K e y s . B a s e F o n t ) ;   } 
 
             s e t   {   E l e m e n t s . S e t N a m e ( K e y s . B a s e F o n t ,   v a l u e ) ;   } 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   P r e p a r e s   t h e   o b j e c t   t o   g e t   s a v e d . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   o v e r r i d e   v o i d   P r e p a r e F o r S a v e ( ) 
 
         { 
 
             b a s e . P r e p a r e F o r S a v e ( ) ; 
 
 
 
 # i f   D E B U G 
 
             i f   ( t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . l o c a   = =   n u l l ) 
 
             { 
 
                 G e t T y p e ( ) ; 
 
             } 
 
 # e n d i f 
 
             / /   C I D   f o n t s   m u s t   b e   a l w a y s   e m b e d d e d .   P D F s h a r p   e m b e d d s   a u t o m a t i c a l l y   a   s u b s e t . 
 
             F o n t D a t a   s u b S e t   =   n u l l ; 
 
             i f   ( t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . l o c a   = =   n u l l ) 
 
                 s u b S e t   =   t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a ; 
 
             e l s e 
 
                 s u b S e t   =   t h i s . f o n t D e s c r i p t o r . d e s c r i p t o r . f o n t D a t a . C r e a t e F o n t S u b S e t ( t h i s . c m a p I n f o . G l y p h I n d i c e s ,   t r u e ) ; 
 
             b y t e [ ]   f o n t D a t a   =   s u b S e t . D a t a ; 
 
 # i f   D E B U G _ 
 
             T r u e T y p e F o n t S u b S e t   f s s   =   n e w   T r u e T y p e F o n t S u b S e t ( " " ,   t h i s . c m a p I n f o . d e s c r i p t o r . f o n t D a t a ,   t h i s . c m a p I n f o . G l y p h I n d i c e s ,   0 ,   f a l s e ,   f a l s e ) ; 
 
             b y t e [ ]   f o n t S u b S e t   =   f s s . P r o c e s s ( ) ; 
 
             f s s . C o m p a r e B y t e s ( f o n t S u b S e t ,   f o n t P r o g r a m ) ; 
 
 # e n d i f 
 
             P d f D i c t i o n a r y   f o n t S t r e a m   =   n e w   P d f D i c t i o n a r y ( t h i s . O w n e r ) ; 
 
             t h i s . O w n e r . I n t e r n a l s . A d d O b j e c t ( f o n t S t r e a m ) ; 
 
             t h i s . f o n t D e s c r i p t o r . E l e m e n t s [ P d f F o n t D e s c r i p t o r . K e y s . F o n t F i l e 2 ]   =   f o n t S t r e a m . R e f e r e n c e ; 
 
 
 
             f o n t S t r e a m . E l e m e n t s [ " / L e n g t h 1 " ]   =   n e w   P d f I n t e g e r ( f o n t D a t a . L e n g t h ) ; 
 
             i f   ( ! t h i s . O w n e r . O p t i o n s . N o C o m p r e s s i o n ) 
 
             { 
 
                 f o n t D a t a   =   F i l t e r i n g . F l a t e D e c o d e . E n c o d e ( f o n t D a t a ) ; 
 
                 f o n t S t r e a m . E l e m e n t s [ " / F i l t e r " ]   =   n e w   P d f N a m e ( " / F l a t e D e c o d e " ) ; 
 
             } 
 
             f o n t S t r e a m . E l e m e n t s [ " / L e n g t h " ]   =   n e w   P d f I n t e g e r ( f o n t D a t a . L e n g t h ) ; 
 
             f o n t S t r e a m . C r e a t e S t r e a m ( f o n t D a t a ) ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   P r e d e f i n e d   k e y s   o f   t h i s   d i c t i o n a r y . 
 
         / / /   < / s u m m a r y > 
 
         p u b l i c   n e w   s e a l e d   c l a s s   K e y s   :   P d f F o n t . K e y s 
 
         { 
 
             / / /   < s u m m a r y > 
 
             / / /   ( R e q u i r e d )   T h e   t y p e   o f   P D F   o b j e c t   t h a t   t h i s   d i c t i o n a r y   d e s c r i b e s ; 
 
             / / /   m u s t   b e   F o n t   f o r   a   C I D F o n t   d i c t i o n a r y . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . N a m e   |   K e y T y p e . R e q u i r e d ,   F i x e d V a l u e   =   " F o n t " ) ] 
 
             p u b l i c   n e w   c o n s t   s t r i n g   T y p e   =   " / T y p e " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( R e q u i r e d )   T h e   t y p e   o f   C I D F o n t ;   C I D F o n t T y p e 0   o r   C I D F o n t T y p e 2 . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . N a m e   |   K e y T y p e . R e q u i r e d ) ] 
 
             p u b l i c   n e w   c o n s t   s t r i n g   S u b t y p e   =   " / S u b t y p e " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( R e q u i r e d )   T h e   P o s t S c r i p t   n a m e   o f   t h e   C I D F o n t .   F o r   T y p e   0   C I D F o n t s ,   t h i s 
 
             / / /   i s   u s u a l l y   t h e   v a l u e   o f   t h e   C I D F o n t N a m e   e n t r y   i n   t h e   C I D F o n t   p r o g r a m .   F o r 
 
             / / /   T y p e   2   C I D F o n t s ,   i t   i s   d e r i v e d   t h e   s a m e   w a y   a s   f o r   a   s i m p l e   T r u e T y p e   f o n t ; 
 
             / / /   I n   e i t h e r   c a s e ,   t h e   n a m e   c a n   h a v e   a   s u b s e t   p r e f i x   i f   a p p r o p r i a t e . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . N a m e   |   K e y T y p e . R e q u i r e d ) ] 
 
             p u b l i c   n e w   c o n s t   s t r i n g   B a s e F o n t   =   " / B a s e F o n t " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( R e q u i r e d )   A   d i c t i o n a r y   c o n t a i n i n g   e n t r i e s   t h a t   d e f i n e   t h e   c h a r a c t e r   c o l l e c t i o n 
 
             / / /   o f   t h e   C I D F o n t . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . D i c t i o n a r y   |   K e y T y p e . R e q u i r e d ) ] 
 
             p u b l i c   c o n s t   s t r i n g   C I D S y s t e m I n f o   =   " / C I D S y s t e m I n f o " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( R e q u i r e d ;   m u s t   b e   a n   i n d i r e c t   r e f e r e n c e )   A   f o n t   d e s c r i p t o r   d e s c r i b i n g   t h e 
 
             / / /   C I D F o n t  s   d e f a u l t   m e t r i c s   o t h e r   t h a n   i t s   g l y p h   w i d t h s . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . D i c t i o n a r y   |   K e y T y p e . M u s t B e I n d i r e c t ,   t y p e o f ( P d f F o n t D e s c r i p t o r ) ) ] 
 
             p u b l i c   n e w   c o n s t   s t r i n g   F o n t D e s c r i p t o r   =   " / F o n t D e s c r i p t o r " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( O p t i o n a l )   T h e   d e f a u l t   w i d t h   f o r   g l y p h s   i n   t h e   C I D F o n t . 
 
             / / /   D e f a u l t   v a l u e :   1 0 0 0 . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . I n t e g e r ) ] 
 
             p u b l i c   c o n s t   s t r i n g   D W   =   " / D W " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( O p t i o n a l )   A   d e s c r i p t i o n   o f   t h e   w i d t h s   f o r   t h e   g l y p h s   i n   t h e   C I D F o n t .   T h e 
 
             / / /   a r r a y  s   e l e m e n t s   h a v e   a   v a r i a b l e   f o r m a t   t h a t   c a n   s p e c i f y   i n d i v i d u a l   w i d t h s 
 
             / / /   f o r   c o n s e c u t i v e   C I D s   o r   o n e   w i d t h   f o r   a   r a n g e   o f   C I D s . 
 
             / / /   D e f a u l t   v a l u e :   n o n e   ( t h e   D W   v a l u e   i s   u s e d   f o r   a l l   g l y p h s ) . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . A r r a y ,   t y p e o f ( P d f A r r a y ) ) ] 
 
             p u b l i c   c o n s t   s t r i n g   W   =   " / W " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( O p t i o n a l ;   a p p l i e s   o n l y   t o   C I D F o n t s   u s e d   f o r   v e r t i c a l   w r i t i n g )   A n   a r r a y   o f   t w o 
 
             / / /   n u m b e r s   s p e c i f y i n g   t h e   d e f a u l t   m e t r i c s   f o r   v e r t i c a l   w r i t i n g . 
 
             / / /   D e f a u l t   v a l u e :   [ 8 8 0   "1 0 0 0 ] . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . A r r a y ) ] 
 
             p u b l i c   c o n s t   s t r i n g   D W 2   =   " / D W 2 " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( O p t i o n a l ;   a p p l i e s   o n l y   t o   C I D F o n t s   u s e d   f o r   v e r t i c a l   w r i t i n g )   A   d e s c r i p t i o n 
 
             / / /   o f   t h e   m e t r i c s   f o r   v e r t i c a l   w r i t i n g   f o r   t h e   g l y p h s   i n   t h e   C I D F o n t . 
 
             / / /   D e f a u l t   v a l u e :   n o n e   ( t h e   D W 2   v a l u e   i s   u s e d   f o r   a l l   g l y p h s ) . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . A r r a y ,   t y p e o f ( P d f A r r a y ) ) ] 
 
             p u b l i c   c o n s t   s t r i n g   W 2   =   " / W 2 " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   ( O p t i o n a l ;   T y p e   2   C I D F o n t s   o n l y )   A   s p e c i f i c a t i o n   o f   t h e   m a p p i n g   f r o m   C I D s 
 
             / / /   t o   g l y p h   i n d i c e s .   I f   t h e   v a l u e   i s   a   s t r e a m ,   t h e   b y t e s   i n   t h e   s t r e a m   c o n t a i n   t h e 
 
             / / /   m a p p i n g   f r o m   C I D s   t o   g l y p h   i n d i c e s :   t h e   g l y p h   i n d e x   f o r   a   p a r t i c u l a r   C I D 
 
             / / /   v a l u e   c   i s   a   2 - b y t e   v a l u e   s t o r e d   i n   b y t e s   2      c   a n d   2      c   +   1 ,   w h e r e   t h e   f i r s t 
 
             / / /   b y t e   i s   t h e   h i g h - o r d e r   b y t e .   I f   t h e   v a l u e   o f   C I D T o G I D M a p   i s   a   n a m e ,   i t   m u s t 
 
             / / /   b e   I d e n t i t y ,   i n d i c a t i n g   t h a t   t h e   m a p p i n g   b e t w e e n   C I D s   a n d   g l y p h   i n d i c e s   i s 
 
             / / /   t h e   i d e n t i t y   m a p p i n g . 
 
             / / /   D e f a u l t   v a l u e :   I d e n t i t y . 
 
             / / /   T h i s   e n t r y   m a y   a p p e a r   o n l y   i n   a   T y p e   2   C I D F o n t   w h o s e   a s s o c i a t e d   T r u e - T y p e   f o n t   
 
             / / /   p r o g r a m   i s   e m b e d d e d   i n   t h e   P D F   f i l e . 
 
             / / /   < / s u m m a r y > 
 
             [ K e y I n f o ( K e y T y p e . D i c t i o n a r y   |   K e y T y p e . S t r e a m O r N a m e ) ] 
 
             p u b l i c   c o n s t   s t r i n g   C I D T o G I D M a p   =   " / C I D T o G I D M a p " ; 
 
 
 
             / / /   < s u m m a r y > 
 
             / / /   G e t s   t h e   K e y s M e t a   f o r   t h e s e   k e y s . 
 
             / / /   < / s u m m a r y > 
 
             i n t e r n a l   s t a t i c   D i c t i o n a r y M e t a   M e t a 
 
             { 
 
                 g e t 
 
                 { 
 
                     i f   ( K e y s . m e t a   = =   n u l l ) 
 
                         K e y s . m e t a   =   C r e a t e M e t a ( t y p e o f ( K e y s ) ) ; 
 
                     r e t u r n   K e y s . m e t a ; 
 
                 } 
 
             } 
 
             s t a t i c   D i c t i o n a r y M e t a   m e t a ; 
 
         } 
 
 
 
         / / /   < s u m m a r y > 
 
         / / /   G e t s   t h e   K e y s M e t a   o f   t h i s   d i c t i o n a r y   t y p e . 
 
         / / /   < / s u m m a r y > 
 
         i n t e r n a l   o v e r r i d e   D i c t i o n a r y M e t a   M e t a 
 
         { 
 
             g e t   {   r e t u r n   K e y s . M e t a ;   } 
 
         } 
 
     } 
 
 } 
 
 